#!/usr/bin/perl
#
# Perl Script for the PMI - this script should be added to your system's crontab. All
# it does is cycle through all the events in the event queue and emails people who 
# have subscribed to those notifications.
#
# Project Homepage: http://majordojo.com/pmi/
# 
# Author: Byrne Reese <byrne@majordojo.com>
#
# Add this line to your crontab to run this update every 10 minutes:
# 0,10,20,30,40,50 * * * * $PMI_HOME/bin/email_events.pl 2>&1
#
#####################################################################################
# CONFIG PARAMS
#####################################################################################

$DB_HOST  = "localhost";
$DB_USER  = "root";
$DB_PASS  = "12ball";
$DB_NAME  = "pmidb";
$USERS_DB = "intranetdb";

$PMI_ADMIN_NAME  = "PMI daemon";
$PMI_ADMIN_EMAIL = "pmi-admin\@intranet.grandcentral.com";

$PMI_BASE_URL = "http://intranet.grandcentral.com/projects";

$SMTP_SERVER = "localhost"; 

$EMAIL_FOOTER = <<EOE;

The following email was sent to you by your own request. If you do not wish to 
receive this email in the future, then please change your preferences on the "My PMI"
dashboard: $PMI_BASE_URL/my/
EOE

#####################################################################################
# DO NOT EDIT BELOW THIS LINE
#####################################################################################
use DBI;
use Mail::Sendmail;

my $pmi_dbh = DBI->connect("DBI:mysql:$DB_NAME:$DB_HOST","$DB_USER","$DB_PASS",{RaiseError => 1}) 
    or die "connecting : $DBI->errstr\n";

$pmi_dbh->prepare("
LOCK TABLES events e WRITE, 
            projects p READ, 
            categories c READ,
            category_watch READ,
            project_watch READ,
            $USERS_DB.Users u READ,
            events WRITE
")
    ->execute or die "Could not lock tables";

# This is hackish, but for starters each event type email will have to be handled 
# separately. First, lets process project events...
#

%EVENTS = build_event_list();
foreach $e (keys %EVENTS) {
    my ($type,$key) = split(":",$e);
    $events = "\n* " . join("\n* ",@{$EVENTS{$e}}); 

    $email_content = "";
    if ($type eq "project") {
	# compose project updated email 
	my $sql = "
SELECT project_id,name
  FROM projects p 
 WHERE p.project_id=$key";
	my $sth = $pmi_dbh->prepare($sql);
	$sth->execute;
	($id,$name) = $sth->fetchrow_array() or break;
	$email_subject = "$name updated";
        $email_content = <<EOE;
This email is to notify you that the project '$name' has been updated. The
following changes have been recorded for the project:
$events

To view this project, please visit its homepage:
$PMI_BASE_URL/projects/project.php?id=$id
EOE
    } elsif ($type eq "category") {
	# compose category updated email 
	my $sql = "
SELECT category_id,name,full_name
  FROM categories c 
 WHERE c.category_id=$key";
	my $sth = $pmi_dbh->prepare($sql);
	$sth->execute;
	($id,$name,$full_name) = $sth->fetchrow_array() or break;
	$email_subject = "Category '$name' updated";
        $email_content = <<EOE;
This email is to notify you that the category '$full_name' has been updated. The
following changes have been recorded for the category:
$events

To view this category, please visit go to:
$PMI_BASE_URL/directory/category.php?id=$id
EOE
    }
    $email_content .= $EMAIL_FOOTER;

    my $sql = "
SELECT userId,email
  FROM ${type}_watch, $USERS_DB.Users u  
 WHERE ${type}_watch.user_id=u.userId
       AND ${type}_watch.${type}_id=$key";
    my $sth = $pmi_dbh->prepare($sql);
    $sth->execute;
    while (($uid,$uemail) = $sth->fetchrow_array()) {
	print "Emailing $uemail these events: $events\n";
	%mail = (To      => $uemail,
		 From    => "$PMI_ADMIN_NAME <$PMI_ADMIN_EMAIL>",
		 Subject => $email_subject,
		 Message => $email_content);
	$mail{smtp} = $SMTP_SERVER;
	sendmail(%mail) or die $Mail::Sendmail::error;	
    } 
}

$pmi_dbh->prepare("DELETE FROM events")->execute;
$pmi_dbh->prepare("UNLOCK TABLES")->execute;

sub build_event_list {
    my %EVENTS;
    my $sql = "
   SELECT DISTINCT foreign_key,event_type,event_description 
     FROM events e
 ORDER BY event_type,foreign_key";
    my $sth = $pmi_dbh->prepare($sql);
    $sth->execute;
    do {
	push @{$EVENTS{"$type:$key"}},$event if (defined($event));
    } while ($key,$type,$event) = $sth->fetchrow_array();
    return %EVENTS;
}

$pmi_dbh->disconnect();

1;

